'use client';
import { useState, useEffect, useRef } from 'react';
import { useAuth, useTVMode } from '@/components/context';
import {
  getRecentUsersWithStatuses,
  updateStatuses,
  updateUserStatus,
  type UserWithStatus,
} from '@/lib/hooks';
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from '@/components/ui';
import { StatusMessage, SubmitButton } from '@/components/default';
import { toast } from 'sonner';
import { HistoryDrawer } from '@/components/status';
import type { Profile } from '@/utils/supabase';
import { makeConditionalClassName } from '@/lib/utils';
import { Card, CardContent, CardHeader } from '@/components/ui/card';
import { Badge } from '@/components/ui/badge';
import { Checkbox } from '@/components/ui/checkbox';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { RefreshCw, Clock, Wifi, WifiOff } from 'lucide-react';
import { useQuery, useMutation, useQueryClient } from '@tanstack/react-query';
import { QueryErrorCodes } from '@/components/context';
import { createClient } from '@/utils/supabase';
import type { RealtimeChannel } from '@supabase/supabase-js';

type ListProps = { initialStatuses: UserWithStatus[] };

export const StatusList = ({ initialStatuses = [] }: ListProps) => {
  const { isAuthenticated } = useAuth();
  const { tvMode } = useTVMode();
  const queryClient = useQueryClient();

  const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  const [selectAll, setSelectAll] = useState(false);
  const [statusInput, setStatusInput] = useState('');
  const [selectedHistoryUser, setSelectedHistoryUser] =
    useState<Profile | null>(null);
  const [updateStatusMessage, setUpdateStatusMessage] = useState('');

  const [connectionStatus, setConnectionStatus] = useState<
    'connecting' | 'connected' | 'disconnected' | 'updating'
  >('connecting');
  const [newStatuses, setNewStatuses] = useState<Set<UserWithStatus>>(new Set());
  const channelRef = useRef<RealtimeChannel | null>(null);
  const supabaseRef = useRef(createClient());

  const {
    data: usersWithStatuses = initialStatuses,
    isLoading: loading,
    error,
    refetch,
    isFetching,
    dataUpdatedAt,
  } = useQuery({
    queryKey: ['users-with-statuses'],
    queryFn: async () => {
      try {
        const response = await getRecentUsersWithStatuses();
        if (!response.success) throw new Error(response.error);
        return response.data;
      } catch (error) {
        toast.error(`Error fetching technicians: ${error as Error}`)
        throw error;
      }
    },
    enabled: isAuthenticated,
    refetchInterval: 30000, // 30 Second interval as we should rely on subscription.
    refetchOnWindowFocus: true,
    refetchOnMount: true,
    initialData: initialStatuses,
    meta: { errCode: QueryErrorCodes.USERS_FETCH_FAILED },
  });

  useEffect(() => {
    if (!isAuthenticated) return;

    const maxReconnectAttempts = 3;
    let reconnectAttempts = 0;
    let reconnectTimeout: NodeJS.Timeout;
    let isComponentMounted = true;
    let currentChannel: RealtimeChannel | null = null;

    const setUpRealtimeConnection = () => {
      if (!isComponentMounted) return;

      if (currentChannel) {
        supabaseRef.current.removeChannel(currentChannel)
          .catch((error) => {
            setConnectionStatus('disconnected');
            console.error(`Error unsubscribing: ${error}`);
          })
        currentChannel = null;
      }

      setConnectionStatus('connecting');
      const channel = supabaseRef.current
        .channel('status_updates')
        .on('broadcast', { event: 'status_updated' }, () => {
          refetch().catch((error) => {
            console.error(`Error refetching statuses: ${error as Error}`)
          });
        })
        .subscribe((status) => {
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          if (status === 'SUBSCRIBED') {
            setConnectionStatus('connected');
            reconnectAttempts = 0;
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === 'CHANNEL_ERROR') {
            setConnectionStatus('disconnected');
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === 'CLOSED') {
            setConnectionStatus('disconnected')
            if (isComponentMounted && reconnectAttempts < maxReconnectAttempts) {
              reconnectAttempts++;
              const delay = 2000 * reconnectAttempts;
              console.log(
                `Reconnecting after close.
                  ${reconnectAttempts} attempts of
                  ${maxReconnectAttempts} in ${delay}ms
              `);
              if (reconnectTimeout) clearTimeout(reconnectTimeout);
              reconnectTimeout = setTimeout(() => {
                if (isComponentMounted) setUpRealtimeConnection();
              }, delay);
            } else {
              console.warn('Max reconnection attempts reached or component is not mounted.');
              setConnectionStatus('disconnected');
            }
          }
        });
      currentChannel = channel;
      channelRef.current = channel;
    };

    const initialTimeout = setTimeout(() => {
      if (isComponentMounted) setUpRealtimeConnection();
    }, 1000);

    return () => {
      isComponentMounted = false;
      if (initialTimeout) clearTimeout(initialTimeout);
      if (reconnectTimeout) clearTimeout(reconnectTimeout);
      if (currentChannel) {
        supabaseRef.current.removeChannel(currentChannel)
          .catch((error) => {
            console.error(`Error unsubscribing: ${error as Error}`)
          });
        channelRef.current = null;
      }
      setConnectionStatus('disconnected');
    }
  }, [isAuthenticated, refetch]);

  const updateStatusMutation = useMutation({

    mutationFn: async ({
      usersWithStatuses,
      status,
    }: {
      usersWithStatuses: UserWithStatus[];
      status: string;
    }) => {
      const previousConnectionStatus = connectionStatus;
      try {
        if (usersWithStatuses.length <= 0) {
          const result = await updateUserStatus(status);
          if (!result.success) throw new Error(result.error);
          return result.data;
        } else {
          const result = await updateStatuses(usersWithStatuses, status);
          if (!result.success) throw new Error(result.error);
          return result.data;
        }
      } catch (error) {
        console.error(`Error updating statuses: ${error as Error}`);
      }
    },
    meta: { errCode: QueryErrorCodes.STATUS_UPDATE_FAILED },

    onMutate: async ({ usersWithStatuses, status }) => {
      // Optimistic update logic
      await queryClient.cancelQueries({ queryKey: ['users-with-statuses']} );
      const previousData = queryClient.getQueryData<UserWithStatus[]>([
        'users-with-statuses',
      ]);
      if (previousData && usersWithStatuses.length > 0) {
        const now = new Date().toISOString();
        const optimisticData = previousData.map((userStatus) => {
          if (usersWithStatuses.some((selected) => selected.user.id === userStatus.user.id))
            return { ...userStatus, status, created_at: now};
          return userStatus;
        });
        queryClient.setQueryData(['users-with-statuses'], optimisticData);

        // Add animation to optimisticly updated statuses
        const updatedStatuses = usersWithStatuses;
        setNewStatuses((prev) => new Set([...prev, ...updatedStatuses]))
        setTimeout(() => {
          setNewStatuses((prev) => {
            const updated = new Set(prev);
            updatedStatuses.forEach((updatedStatus) => updated.delete(updatedStatus));
            return updated;
          });
        }, 1000)
      }
      return { previousData };
    },

    onSuccess: (data) => {
      queryClient.invalidateQueries({ queryKey: ['users-with-statuses']} )
        .catch((error) => console.error(`Error invalidating query: ${error}`));
      if (!data) return;
      data.forEach((statusUpdate) => {
        toast.success(`
          ${statusUpdate.user.full_name}'s status updated to '${statusUpdate.status}'.
        `);
      });
      setSelectedUsers([]);
      setStatusInput('');
    },

    onError: (error, _variables, context) => {
      if (context?.previousData)
        queryClient.setQueryData(['users-with-statuses'], context.previousData);
      toast.error(`Error updating statuses: ${error}`);
    },
  });

  const handleUpdateStatus = () => {
    if (!isAuthenticated) {
      setUpdateStatusMessage('You must be signed in to update technican statuses!')
      return;
    } else if (!statusInput.trim()) {
      setUpdateStatusMessage('Your status must be in between 3 & 80 characters long!');
      return;
    }
    updateStatusMutation.mutate({
      usersWithStatuses: selectedUsers,
      status: statusInput.trim(),
    });
  };


  const handleCheckboxChange = (user: UserWithStatus) => {
    setSelectedUsers((prev) =>
      prev.some((u) => u.user.id === user.user.id)
        ? prev.filter((prevUser) => prevUser.user.id !== user.user.id)
        : [...prev, user],
    );
  };

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([]);
    } else {
      setSelectedUsers(usersWithStatuses);
    }
    setSelectAll(!selectAll);
  };

  useEffect(() => {
    setSelectAll(
      selectedUsers.length === usersWithStatuses.length &&
        usersWithStatuses.length > 0,
    );
  }, [selectedUsers.length, usersWithStatuses.length]);

  const getConnectionIcon = () => {
    switch (connectionStatus) {
      case 'connected':
        return <Wifi className='w-4 h-4 text-green-500' />;
      case 'connecting':
        return <Wifi className='w-4 h-4 text-yellow-500 animate-pulse' />;
      case 'disconnected':
        return <WifiOff className='w-4 h-4 text-red-500' />;
      case 'updating':
        return <RefreshCw className='w-4 h-4 text-blue-500 animate-spin' />;
    }
  };

  const getConnectionText = () => {
    switch (connectionStatus) {
      case 'connected':
        return 'Connected';
      case 'connecting':
        return 'Connecting...';
      case 'disconnected':
        return 'Disconnected';
      case 'updating':
        return 'Updating...';
    }
  };

  const formatTime = (timestamp: string) => {
    const date = new Date(timestamp);
    const time = date.toLocaleTimeString('en-US', {
      hour: 'numeric',
      minute: 'numeric',
    });
    const day = date.getDate();
    const month = date.toLocaleString('default', { month: 'long' });
    return `${time} - ${month} ${day}`;
  };

  if (loading) {
    return (
      <div className='flex justify-center items-center min-h-[400px]'>
        <Loading className='w-full' alpha={0.5} />
      </div>
    );
  }

  if (error) {
    return (
      <div className='flex flex-col justify-center items-center min-h-[400px] gap-4'>
        <p className='text-red-500'>Error loading status updates</p>
        <Button onClick={() => refetch()} variant='outline'>
          <RefreshCw className='w-4 h-4 mr-2' />
          Retry
        </Button>
      </div>
    );
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'mx-auto space-y-4',
    on: 'lg:w-11/12 w-full mt-15',
    off: 'w-5/6',
  });

  const headerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: '',
    on: 'none',
    off: 'flex items-center justify-between mb-4',
  })

  const cardClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'transition-all duration-300 hover:shadow-md',
    on: 'lg:text-4xl',
    off: 'lg:text-base',
  });

  return (
    <div className={containerClassName}>
      <div className={headerClassName}>
        <div className='flex items-center gap-2'>
          <Checkbox
            id='select-all'
            checked={selectAll}
            onCheckedChange={handleSelectAllChange}
          />
          <label htmlFor='select-all' className='text-sm font-medium'>
            Select All
          </label>
          <Badge variant='outline' className='flex items-center gap-2'>
            {getConnectionIcon()}
            <span className='text-sm'>{getConnectionText()}</span>
          </Badge>
        </div>
      </div>
    </div>
  );

};
